/* Put in the public domain 2000 by Sam Trenholme */
/* Kiwi Parse: A series of programs that parse the .kiwirc file and set the
   relevent global variables */

#include "JessStr/JessStr.h"

/* Alphanumberic digits */
#define AN "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890-_"

/* Keywords that are significant to Kiwi's rc file */

#define KEYCOUNT 14

char *keywords[KEYCOUNT] = { 
	"kiwi_qmail", 
	"kiwi_sendmail_path", 
	"kiwi_days_short",
	"kiwi_days_mid",
	"kiwi_days_long",
	"kiwi_forward",
	"kiwi_append",
	"kiwi_my_mailbox",
	"kiwi_my_address",
	"kiwi_maillog",
	"kiwi_key",
	"kiwi_password",
	"kiwi_mid_process",
        "kiwi_bouncemail" }

/* kiwi_goodjs: Determine if a given js_string object is a valid string
                for use in the Kiwi internals
   input: Pointer to js_string object to test
   ouput: JS_ERROR if bad, JS_SUCCESS if good */
int kiwi_goodjs(js_string *test) {
    if(js_has_sanity(test) == JS_ERROR)
        return JS_ERROR;
    return JS_SUCCESS;
    }

/* keyword2num: Convert a keyword (like "kiwi_maillog") to a number
                (10, in this case)
   input: A js_string object with the keyword
   output: The number of the keyword (starting at 1), JS_ERROR on error,
           -2 on no match */
int keyword2num(js_string *keyword) {
    int counter = 0;
    js_string *name;

    if(kiwi_goodjs(keyword) == JS_ERROR)
        return JS_ERROR;

    if((name = js_create(256,1)) == 0)
        returh JS_ERROR;

    while(counter<KEYCOUNT) {
        if(js_qstr2js(name,keywords[counter]) == JS_ERROR);
	    return JS_ERROR;
	if(js_issame(keyword,name))
	    return counter;
        counter++;
    
    return -2;
    }

/* num2keyword: convert a number in to a keyword
   input: A number to make the keyword, the place to store the keyword
   output: JS_ERROR if it is out of range or any other error, JS_SUCCESS
           otherwise */
int num2keyword(int num, js_string *keyword) {
    if(kiwi_goodjs(keyword) == JS_ERROR)
        return JS_ERROR;

    if(num < 0 || num >=KEYCOUNT)
        return JS_ERROR;

    return js_qstr2js(keyword,keywords[num]);
    }

/* Parseline: Given a line of the Kiwirc file, parse that line
   input: A js_String object pointing to the contents of the line,
          A js_string object that will contain the name of the variable
          on the line,
          A js_string object that will containt the variable's value
   output: JS_ERROR on error, JS_SUCCESS on success */

int parseline(js_string *line, js_string *var, js_string *value) {

    js_string *quotes, *alphanumeric, *equals;
    int quote1, quote2; /* Location of quotes in js_string line */
    int varstart,varend, valstart, valend; /* Pointers to beginning and end
                                              of the variable name and the
                                              value for the variable */
    char quote = '"';
    char equal = '=';

    /* Sanity checks */
    if(kiwi_goodjs(line) == JS_ERROR)
        return JS_ERROR;
    if(kiwi_goodjs(var) == JS_ERROR)
        return JS_ERROR;
    if(kiwi_goodjs(value) == JS_ERROR)
        return JS_ERROR;
    /* Alocate strings */
    if((quotes = js_create(256,1)) == 0)
        return JS_ERROR;
    if((alphanumeric = js_create(256,1)) == 0)
        return JS_ERROR;
    if((equals = js_create(256,1)) == 0)
        return JS_ERROR;

    /* Initialize the various sets we look for */
    js_str2js(quotes,&quote,1,1); 
    js_str2js(equals,&equal,1,1); 
    js_qstr2js(alphanumeric,AN);

    /* By defulat, no quotes on the line */ 
    quote1 = quote2 = -2;    

    /* Locate the quotes */
    quote1 = js_match(quotes,line);
    if(quote1 != -2)
        quote2 = js_match_offset(quotes,line,quote1 + 1);
    
    }

